'use strict';

require('babel-polyfill');

var _del = require('del');

var _del2 = _interopRequireDefault(_del);

var _parseGithubUrl = require('parse-github-url');

var _parseGithubUrl2 = _interopRequireDefault(_parseGithubUrl);

var _gunzipMaybe = require('gunzip-maybe');

var _gunzipMaybe2 = _interopRequireDefault(_gunzipMaybe);

var _input = require('input');

var _input2 = _interopRequireDefault(_input);

var _meow = require('meow');

var _meow2 = _interopRequireDefault(_meow);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _package = require('../../package.json');

var _package2 = _interopRequireDefault(_package);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _sander = require('sander');

var _sander2 = _interopRequireDefault(_sander);

var _shellEscape = require('shell-escape');

var _shellEscape2 = _interopRequireDefault(_shellEscape);

var _crossSpawn = require('cross-spawn');

var _crossSpawn2 = _interopRequireDefault(_crossSpawn);

var _tarFs = require('tar-fs');

var _tarFs2 = _interopRequireDefault(_tarFs);

var _updateNotifier = require('update-notifier');

var _updateNotifier2 = _interopRequireDefault(_updateNotifier);

var _child_process = require('child_process');

var _chalk = require('chalk');

var _figures = require('figures');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new _bluebird2.default(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return _bluebird2.default.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

var prompt = (0, _chalk.grey)('>');

var help = '\n\t\t🎬  ' + (0, _chalk.cyan)('startfrom') + '\n\n\t\t1. Downloads a snapshot of the specified repo to your current\n\t\t\t working directory\n\t\t2. Does ' + (0, _chalk.cyan)('git init') + ', ' + (0, _chalk.cyan)('git add .') + ' and ' + (0, _chalk.cyan)('git commit') + '\n\t\t3. If there\'s a package.json, runs ' + (0, _chalk.cyan)('npm install') + '\n\n\t\t' + (0, _chalk.grey)('──────────────────────────────────────────────────────────────────────') + '\n\n\t\tUsage\n\t\t\t' + prompt + ' startfrom ' + (0, _chalk.cyan)('user') + '/' + (0, _chalk.cyan)('repo') + (0, _chalk.grey)('[') + '#' + (0, _chalk.cyan)('ref') + (0, _chalk.grey)(']') + ' ' + (0, _chalk.grey)('[') + (0, _chalk.cyan)('subdirectory') + (0, _chalk.grey)(']') + '\n\n\t\tExamples:\n\t\t\t' + prompt + ' startfrom google/web-starter-kit\n\t\t\t' + prompt + ' startfrom google/web-starter-kit#v0.5.4\n\t\t\t' + prompt + ' startfrom h5bp/html5-boilerplate#v5.0.0 dist\n\t\t\t' + prompt + ' startfrom https://github.com/mxstbr/react-boilerplate\n\n\t\tFlags:\n\t\t\t--confirm    Automatically confirm the list of files to include\n';

var cli = (0, _meow2.default)({ help: help, description: false }, {
	default: {
		confirm: false
	}
});

// if no CLI args, do the same as `startfrom --help`
if (!cli.input[0]) {
	console.log(help);
	process.exit(0);
}

var dir = process.cwd();

var _ghParse = (0, _parseGithubUrl2.default)(cli.input[0]);

var owner = _ghParse.owner;
var name = _ghParse.name;
var ref = _ghParse.branch;

var subdirectory = cli.input[1] || '.';

// check for updates
var notifier = (0, _updateNotifier2.default)({ pkg: _package2.default });
notifier.notify();

function run() {
	for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
		args[_key] = arguments[_key];
	}

	return new _bluebird2.default(function (resolve, reject) {
		console.log(' ' + prompt + ' ' + (0, _shellEscape2.default)(args));

		var child = (0, _crossSpawn2.default)(args.shift(), args, {
			stdio: 'inherit',
			cwd: dir
		});

		child.on('error', reject);

		child.on('close', function (code) {
			if (code === 0) resolve();else reject(new Error('Command failed'));
		});
	});
}

// begin the async sequence
_asyncToGenerator(regeneratorRuntime.mark(function _callee2() {
	var files, hasGit, keep;
	return regeneratorRuntime.wrap(function _callee2$(_context2) {
		while (1) {
			switch (_context2.prev = _context2.next) {
				case 0:
					if (!(!owner || !name)) {
						_context2.next = 2;
						break;
					}

					throw new Error('Invalid: ' + cli.input[0]);

				case 2:

					console.log('');

					_context2.next = 5;
					return _sander2.default.readdir(dir);

				case 5:
					files = _context2.sent;


					// the directory must be empty (except for .git and .DS_Store), otherwise we exit
					if (files.filter(function (file) {
						return file !== '.DS_Store' && file !== '.git';
					}).length) {
						console.log((0, _chalk.red)('This directory is not empty!'));
						console.log('\nPlease ' + (0, _chalk.cyan)('cd') + ' into an empty directory and try again.');
						process.exit(1);
					}

					// if the directory is already git-managed, that is ok providing there are no
					// uncommitted changes (this covers the scenario where user has deleted all
					// existing files from an existing project in order to do a 'fresh start'
					// with the starter kit while retaining history)
					hasGit = files.indexOf('.git') > -1;

					if (hasGit) {
						say('This is alreay a git-managed directory; verifying working directory is clean...');

						try {
							(0, _child_process.execSync)('git diff --exit-code');
						} catch (error) {
							console.log((0, _chalk.red)('\nWorking directory is not clean.'));
							console.log('Please commit your changes then try running this script again.');
							process.exit(1);
						}
						doTick();
					}

					// download whole repo tarball into ./__startfrom_tmp
					_context2.next = 11;
					return new _bluebird2.default(function (resolve, reject) {
						var tarballURL = 'https://github.com/' + owner + '/' + name + '/archive/' + ref + '.tar.gz';
						say('Downloading and extracting ' + tarballURL);

						(0, _request2.default)(tarballURL).pipe((0, _gunzipMaybe2.default)()).pipe(_tarFs2.default.extract(dir, {
							map: function map(header) {
								header.name = '__startfrom_tmp/' + header.name.substring(header.name.indexOf('/') + 1);
							}
						}).on('finish', resolve)).on('error', reject);
					});

				case 11:

					// decide which files should be kept
					keep = void 0;
					return _context2.delegateYield(regeneratorRuntime.mark(function _callee() {
						var tmpDir, choices;
						return regeneratorRuntime.wrap(function _callee$(_context) {
							while (1) {
								switch (_context.prev = _context.next) {
									case 0:
										// make a list of choices (root-level files/folders)
										tmpDir = _path2.default.resolve(dir, '__startfrom_tmp', subdirectory);
										_context.next = 3;
										return _bluebird2.default.map(_sander2.default.readdir(tmpDir), function (file) {
											return _sander2.default.stat(tmpDir, file).then(function (stat) {
												var choice = { value: file };

												if (stat.isDirectory()) choice.name = (0, _chalk.cyan)(' ' + file + '/');else choice.name = (0, _chalk.cyan)(' ' + file);

												switch (file.toLowerCase()) {
													case 'docs':
													case 'doc':
													case 'license':
													case 'license.md':
													case 'licence':
													case 'licence.md':
													case 'readme.md':
													case 'readme':
													case 'changelog.md':
													case 'changelog':
														choice.checked = false;
														break;
													default:
														choice.checked = true;
												}

												return choice;
											});
										});

									case 3:
										choices = _context.sent;


										// sort them - unchecked items first, then sort alphabetically
										choices.sort(function (a, b) {
											if (a.checked && b.checked) {
												if (a.name > b.name) return 1;
												if (a.name < b.name) return -1;
												return 0;
											}
											if (a.checked) return 1;
											return -1;
										});

										if (!cli.flags.confirm) {
											_context.next = 9;
											break;
										}

										keep = choices.map(function (choice) {
											return choice.value;
										});
										_context.next = 13;
										break;

									case 9:
										// let the user select the files to keep
										console.log();
										_context.next = 12;
										return _input2.default.checkboxes('Which files and folders do you want?', choices);

									case 12:
										keep = _context.sent;

									case 13:
									case 'end':
										return _context.stop();
								}
							}
						}, _callee, undefined);
					})(), 't0', 13);

				case 13:
					_context2.next = 15;
					return _bluebird2.default.each(keep, function (file) {
						return _sander2.default.rename(dir, '__startfrom_tmp', subdirectory, file).to(dir, file);
					});

				case 15:
					_context2.next = 17;
					return (0, _del2.default)(_path2.default.resolve(dir, '__startfrom_tmp'));

				case 17:

					// commit everything...
					say('Committing initial state...');

					if (hasGit) {
						_context2.next = 21;
						break;
					}

					_context2.next = 21;
					return run('git', 'init');

				case 21:
					_context2.next = 23;
					return run('git', 'add', '.');

				case 23:
					_context2.next = 25;
					return run('git', 'commit', '-m', 'startfrom ' + (0, _shellEscape2.default)(process.argv.slice(2)));

				case 25:
					doTick();

					// run npm install if there's a package.json
					_context2.next = 28;
					return _sander2.default.exists(dir, 'package.json');

				case 28:
					if (!_context2.sent) {
						_context2.next = 33;
						break;
					}

					say('Running npm install for you (this might take a while)...');
					_context2.next = 32;
					return run('npm', 'install');

				case 32:
					doTick();

				case 33:

					console.log('\n' + '\n  ' + (0, _chalk.bgWhite)('                         ') + '\n  ' + _chalk.bgWhite.black('  🎬  startfrom complete  ') + '\n  ' + (0, _chalk.bgWhite)('                         ') + '\n');

				case 34:
				case 'end':
					return _context2.stop();
			}
		}
	}, _callee2, undefined);
}))();

function say(message) {
	console.log((0, _chalk.yellow)('\n\n' + message));
}

function doTick() {
	console.log((0, _chalk.green)(_figures.tick));
}
//# sourceMappingURL=index.js.map